#!/usr/bin/perl

require 5;

package Exim;

use strict;  # Be strict !
use integer; # Be fast !
use Carp;
use File::Basename;
use FileHandle;

sub _init_exec_
  {
    my $self = shift;
    my $exec_path = shift;

    if ($exec_path) {
      substr($exec_path, 0, 1) eq '/'
      and (basename($exec_path) eq 'exim')
      and -x $exec_path   # Must be executable by the effective uid/gid.
      or return undef;
    }
    else { $exec_path = 'exim' }
    $$self{exec} = $exec_path;
    version($self)
  }

sub _init_conf_
  {
    my $self = shift;
    my $config_file = shift;

    if ($config_file) {
      $config_file or croak("Unable to read config file: $config_file");
      $$self{conf} = $config_file;
    }
    else { config_file($self) }
  }

# Exim->new(exec => '/usr/sbin/exim', conf => '/etc/exim.conf')
# or
# Exim->new() which try to determine default values
sub new
  {
    my $class = shift;
    $class = ref($class) || $class;
    my %params = @_;

    my $self = {};

    _init_exec_ ($self, $params{exec}) or return undef;
    _init_conf_ ($self, $params{conf}) or return undef;

    bless $self, $class ;
  }

sub output
  {
    my $self = shift;

    $$self{ouput} = shift if @_;
    return $$self{output};
  }

sub result
  {
    my $self = shift;

    $$self{result} = shift if @_;
    return $$self{result};
  }

# $exim->_exec(-bP => 'configure_file' )
sub _exec
  {
    my $self = shift;
    @_ or carp '$self->error(no_clo)', return undef;


    my $command = $$self{exec};
    $command .= ' -C'.$$self{conf_path} if defined $$self{conf_path};
    $command .= ' ';
    $command .= join ' ', @_;

    return $command if $$self{result} eq 'command';

    if ($$self{output} eq 'all') {
      $command .= ' 2>&1'
    }
    elsif ($$self{output} eq 'errors_only') {
      $command .= ' 2>&1 1>/dev/null'
    }
    elsif ($$self{output} eq 'nothing') {
      $command .= ' &>/dev/null'
    }
    else { $command .= ' 2>/dev/null' }

    my $pipe = FileHandle->new($command.'|');

    return $pipe if $$self{result} eq 'pipe';

    my @lines = $pipe->getlines;
    chomp @lines;

    $pipe->close or carp '$self->error(child)', return undef;

    return join "\n", @lines if $$self{result} eq 'buffer';

    return @lines;
  }

sub option
  {
    my $self = shift;
    (@_ == 1) or return undef;
    my $option = shift;

    my $result = $$self{result};
    $$self{result} = 'buffer';
    my $buffer = _exec($self, -bP => $option);
    chomp $buffer;
    $$self{result} = $result;

    if ($buffer eq $option) {
      return 1;
    }
    elsif ($buffer eq "no_$option") {
      return 0;
    }
    elsif ($buffer =~ /=/) {
      return ($buffer =~ /=\s(.*)$/);
    }
    else {
      return undef;
    }
  }

# Return the current version of exim
# version()
sub version
  {
    my $self = shift;

    if (not defined $$self{version}) {
      my $result = $$self{result};
      $$self{result} = 'buffer';
      my $output = _exec($self, '-bV');
      $$self{result} = $result;
      ($$self{version}) = ($output =~ /^Exim version (\d+\.\d+)/);
    }
    return $$self{version};
  }

# Return the default configuration file.
# $exim->config_file()
sub config_file
  {
    my $self = shift;
    if (not defined $$self{conf}) {
      my $result = $$self{result};
      $$self{result} = 'buffer';
      $$self{conf} = _exec ($self, -bP => 'configure_file' );
      $$self{result} = $result;
    }	
    return $$self{conf};
  }

# Returns the path to the exim log files
# where '%s' must be replaced by 'main', 'panic' or 'reject'
# $exim->log_files_path()
sub log_files_path
  {
    my $self = shift;
    $$self{output} = 'all';
    my @line = option ($self, 'log_file_path');

    return pop @line
  }

sub queue_run
  {
    my $self = shift;
    my $type = shift; # q, R, S
    my $flags = shift;

   _exec($self, "-$type$flags", @_);
  }

1;
